/*
 * File:   mainwindow.cpp
 * Author: stefan
 *
 * Created on 29. Dezember 2011, 12:58
 */

#include "mainwindow.h"

#include <QSplitter>
#include <QPlainTextEdit>
#include <QApplication>
#include <QMenu>
#include <QMessageBox>
#include <QCloseEvent>
#include <QTimer>
#include <QMenuBar>
#include <QFileDialog>
#include <QScrollArea>
#include <QLabel>
#include <QAction>
#include <QDir>
#include <QtDebug>

MainWindow::MainWindow()
{

  setWindowTitle(tr("Steganografie [*]"));

  initView();
  initMenu();

  QTimer::singleShot(100, this, SLOT(fileOpen()));
}

MainWindow::~MainWindow()
{
}

void MainWindow::initView()
{
  m_splitter=new QSplitter(Qt::Vertical, this);
  setCentralWidget(m_splitter);

  m_area=new QScrollArea(this);
  m_area->setBackgroundRole(QPalette::Dark);
  m_splitter->addWidget(m_area);

  m_label=new QLabel(m_area);
  m_label->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
  m_label->setScaledContents(false);
  m_area->setWidget(m_label);

  m_edit=new QPlainTextEdit(m_splitter);
  m_splitter->addWidget(m_edit);
  connect(m_edit, SIGNAL(textChanged()), this, SLOT(textChanged()));
}

void MainWindow::initMenu()
{
  m_open=new QAction(tr("&Open..."), this);
  m_open->setShortcut(QKeySequence::Open);
  connect(m_open, SIGNAL(triggered()), this, SLOT(fileOpen()));

  m_save=new QAction(tr("&Save"), this);
  m_save->setShortcut(QKeySequence::Save);
  connect(m_save, SIGNAL(triggered()), this, SLOT(fileSave()));

  m_saveAs=new QAction(tr("Save &as..."), this);
  m_saveAs->setShortcut(QKeySequence::SaveAs);
  connect(m_saveAs, SIGNAL(triggered()), this, SLOT(fileSaveAs()));

  m_quit=new QAction(tr("&Quit..."), this);
  m_quit->setShortcut(QKeySequence::Quit);
  connect(m_quit, SIGNAL(triggered()), qApp, SLOT(quit()));

  QMenu *menu=menuBar()->addMenu(tr("&File"));
  menu->addAction(m_open);
  menu->addSeparator();
  menu->addAction(m_save);
  menu->addAction(m_saveAs);
  menu->addSeparator();
  menu->addAction(m_quit);
}

void MainWindow::fileOpen()
{
  static QString dir=QDir::homePath();

  QString filter;
  filter=tr("Images (*.png *.xpm *.jpg *.bmp)");

  QString filename=QFileDialog::getOpenFileName(this, tr("Open Image"), dir, filter);

  if (!filename.isEmpty()) {
    QImage img;
    if (img.load(filename)) {
      m_image=img;
      m_filename=filename;
      setWindowModified(false);

      m_label->setPixmap(QPixmap::fromImage(img));

      QString txt=readTextFromImage(img);
      m_edit->setPlainText(txt);
    }
  }

  if (m_image.isNull()) {
    close();
  }
}

void MainWindow::fileSave()
{

}

void MainWindow::fileSaveAs()
{
  QString filter;
  filter=tr("Images (*.png *.xpm *.jpg *.bmp)");

  QString filename=QFileDialog::getSaveFileName(this, tr("Open Image"), m_filename, filter);

  if (!filename.isEmpty()) {
    const QString txt=m_edit->toPlainText();
    QImage img=writeTextIntoImage(m_image, txt);
    if (img.save(filename)) {
      setWindowModified(false);
    }
  }
}

void MainWindow::closeEvent(QCloseEvent *event)
{
  if (isWindowModified()) {
    if (QMessageBox::question(this, tr("Quit?"), tr("Do you really want to quit without saving?"), QMessageBox::Yes|QMessageBox::No, QMessageBox::No)==QMessageBox::No) {
      event->ignore();
    } else {
      event->accept();
    }
  } else {
    event->accept();
  }
}

QString MainWindow::readTextFromImage(const QImage &img) const
{
  const int width=img.width();
  const int height=img.height();

  QByteArray data;

  int byte=0;
  int bits=0;

  for (int y=0; y<height; ++y) {
    for (int x=0; x<width; ++x) {
      QRgb rgb=img.pixel(x, y);

      byte|=((rgb>>16) & 1) << bits;
      ++bits;
      if (bits==8) {
        data.append(byte);
        byte=0;
        bits=0;
      }

      byte|=((rgb>>8) & 1) << bits;
      ++bits;
      if (bits==8) {
        data.append(byte);
        byte=0;
        bits=0;
      }

      byte|=(rgb & 1) << bits;
      ++bits;
      if (bits==8) {
        data.append(byte);
        byte=0;
        bits=0;
      }
    }
  }

  return QString(data);
}

QImage MainWindow::writeTextIntoImage(const QImage &src, const QString &text) const
{
  const QByteArray data=text.toAscii();
  const int width=src.width();
  const int height=src.height();
  QImage dst;
  dst=QImage(width, height, QImage::Format_ARGB32);

  if (!data.isEmpty()) {
    int byte=0;
    int bits=0;
    int pos=0;

    byte=data.at(pos);

    for (int y=0; y<height; ++y) {
      for (int x=0; x<width; ++x) {
        QRgb rgb=src.pixel(x, y);

        rgb&=0xfffefefe;

        rgb|=(byte & 1) << 16;
        byte>>=1;
        ++bits;
        if (bits==8) {
          ++pos;
          if (pos<data.size()) {
            byte=data.at(pos);
          } else {
            byte=0;
          }
          bits=0;
        }

        rgb|=(byte & 1) << 8;
        byte>>=1;
        ++bits;
        if (bits==8) {
          ++pos;
          if (pos<data.size()) {
            byte=data.at(pos);
          } else {
            byte=0;
          }
          bits=0;
        }

        rgb|=byte & 1;
        byte>>=1;
        ++bits;
        if (bits==8) {
          ++pos;
          if (pos<data.size()) {
            byte=data.at(pos);
          } else {
            byte=0;
          }
          bits=0;
        }

        dst.setPixel(x, y, rgb);
      }
    }
  }

  return dst;
}

void MainWindow::textChanged()
{
  if (!m_image.isNull()) {
    //const QString txt=m_edit->toPlainText();
    //QImage img=writeTextIntoImage(m_image, txt);

    setWindowModified(true);

    //m_image=img;
    //m_label->setPixmap(QPixmap::fromImage(img));
  }
}

